/* TGI Test Program for cc65 by Fatih Aygun */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <tgi.h>

static unsigned int memory_footprint;
static unsigned int x_res, y_res;
static unsigned char color_count, page_count;
// static unsigned char font_x, font_y;

static void check_error (const char* s){
	unsigned char error = tgi_geterror ();
	if (error != TGI_ERR_OK) {
		tgi_done();
		printf ("\n\n%s error %d\n", s, error);
		exit(-1);
    }
}

static void load_driver(void)
{
	char buffer[20], *driver_name;

	puts("\n\ntgi driver to test?");
	gets(buffer);

	memory_footprint = _heapmemavail();
	
	driver_name = buffer;
	while (*(driver_name) == ' ') {
		++driver_name;
	}

	if (*driver_name == 0) {
		printf("\nLoading default: '%s'\n", tgi_stddrv);
		tgi_load_driver (tgi_stddrv);
	} else {
		printf("\nLoading '%s'\n", driver_name);
		tgi_load_driver (driver_name);
	}

	check_error("tgi_load_driver");
	puts("Driver loaded successfully");
	memory_footprint -= _heapmemavail();
}

static void show_driver_properties(void)
{
	printf("\nx resolution: %d\n", x_res = tgi_getxres());
	printf("y resolution: %d\n", y_res = tgi_getyres());
	printf("color count: %d\n", color_count = tgi_getcolorcount());
	printf("page count: %d\n", page_count = tgi_getpagecount());

//	tgi_textstyle(0x100, 0x100, TGI_TEXT_HORIZONTAL, TGI_FONT_BITMAP);

//	printf("font width: %d\n", font_x = tgi_textwidth("M"));
//	printf("font height: %d\n", font_y = tgi_textheight("M"));
	printf("memory footprint: %d\n", memory_footprint);
	puts("");
	puts("Press any key to continue");
	cgetc();
}

static void test_set_get(void)
{
	int x;
	int y;
	unsigned char c1, c2;

	puts("\n\nThis test will plot random points on");
	puts("the screen. You can stop the test by");
	puts("pressing a key.");
	puts("");
	puts("Press any key to continue");
	cgetc();
    puts("Now starting tgi_init()"); // TEST
    cgetc(); // TEST
	tgi_init();
    puts("After tgi_init()"); // TEST
    cgetc(); // TEST
	check_error("tgi_init");
    puts("After check_error()"); // TEST
    cgetc(); // TEST

	// Do the test

	while (!kbhit()) {
		x = rand() % x_res;
		y = rand() % y_res;
		c1 = rand() % color_count;
		tgi_setcolor(c1);
		tgi_setpixel(x, y);
		c2 = tgi_getpixel(x, y);
		if (c1 != c2) {
			tgi_done();
			clrscr();
			printf("Test failed: x=%d, y=%d, c1=%d, c2=%d", x, y, c1, c2);
			exit(-1);
		}

	}
	cgetc();
	tgi_done();
}

int main(void)
{
    load_driver();
    show_driver_properties();
    _randomize();
    test_set_get();
    tgi_unload();

	return EXIT_SUCCESS;
}
